package com.semarchy.mdm.client.samples;

import java.util.Collections;
import java.util.List;

import com.semarchy.mdm.client.IDataEditionConnection;
import com.semarchy.mdm.client.IMDMConnection;
import com.semarchy.mdm.client.MDMConnectionFactory;
import com.semarchy.mdm.runtime.data.DataQuery;
import com.semarchy.mdm.runtime.data.IDataRow;
import com.semarchy.mdm.runtime.data.IEntityRow;
import com.semarchy.mdm.runtime.data.QueryCondition;
import com.semarchy.mdm.runtime.model.dataaccess.AttributeNames;
import com.semarchy.mdm.runtime.model.dataaccess.DataAccessModelConstants;
import com.semarchy.mdm.runtime.model.dataaccess.IDataAccessModel;
import com.semarchy.mdm.runtime.model.dataaccess.IView;
import com.semarchy.mdm.runtime.model.logical.IDisplayViewAttribute;
import com.semarchy.mdm.runtime.model.logical.IApplication;
import com.semarchy.mdm.runtime.model.logical.IBOEntity;
import com.semarchy.mdm.runtime.model.logical.IBOTransition;
import com.semarchy.mdm.runtime.model.logical.IBusinessObject;
import com.semarchy.mdm.runtime.model.logical.IBusinessObjectView;
import com.semarchy.mdm.runtime.model.logical.IEntity;
import com.semarchy.mdm.runtime.model.logical.IFormSection;
import com.semarchy.mdm.runtime.model.logical.IFormView;
import com.semarchy.mdm.runtime.model.logical.IModel;

public class Sample7_BrowseBOView {

	public static void main(String[] args) {
		IMDMConnection conn = MDMConnectionFactory.connect(
				SamplesConstants.SEMARCHY_URL, SamplesConstants.USERNAME,
				SamplesConstants.PASSWORD);

		IDataEditionConnection dataEditionConnection = conn
				.createDataEditionConnection(SamplesConstants.GS_DATAEDITION);

		IModel logicalModel = dataEditionConnection.getLogicalModel();
		IApplication demoApplication = logicalModel
				.getApplication(SamplesConstants.GS_DEMO_APP);

		// Selecting which BO View to Access
		IBusinessObjectView boView = demoApplication
				.getBOView("EmployeesByCostCenterView");
		IBusinessObject bo = boView.getBusinessObject();
		IBOEntity rootBoEntity = bo.getRootBOEntity();
		String boFilter = bo.getFilter();

		int level = 0;
		StringBuilder builder = new StringBuilder();

		// Recursive function call to process BO Entities in cascade
		processBoEntity(dataEditionConnection, rootBoEntity, boFilter, boView, null, null, level, builder);
		
		System.out.println(builder);

	}

	private static void processBoEntity(
			IDataEditionConnection dataEditionConnection, IBOEntity boEntity,
			String boFilter, IBusinessObjectView boView, IBOTransition transition, IDataRow parentRow, int level, StringBuilder builder) {

		
		IDataAccessModel dataAccessModel = dataEditionConnection
				.getDataAccessModel();

		IFormView formView = boView.getBOViewEntity(boEntity).getBrowsingFormView();
		IEntity entity = boEntity.getEntity();
		IView goldenView = dataAccessModel.getGoldenView(entity);
		DataQuery qry = new DataQuery(goldenView);
		
		
		// Build the condition
		QueryCondition transCondition = null;
		if (transition != null) {
			String filter = DataAccessModelConstants.getToGoldenAttributeName(goldenView.getManyToOneNode(transition.getReference().getToRoleName())) + " = :ID";
			transCondition = new QueryCondition(filter, Collections.singletonMap("ID", parentRow.getValue(AttributeNames.goldenIdName(parentRow.getView()))));
		}
		QueryCondition boFilterCond = null;
		if (boFilter != null) {
			boFilterCond = new QueryCondition(boFilter);
		}
		
		// Add to the query BO filtering at each level
		QueryCondition cond = QueryCondition.and(transCondition, boFilterCond);		
		qry.setWhereCondition(cond.getCondition());
		qry.setBindings(cond.getBindings());

		// building the Query based on the form attributes in the BO View's form
		// view

		for (IFormSection formSection : formView.getFormSections()) {
			for (IDisplayViewAttribute formAttribute : formSection.getFormAttributes()) {
				// add the form attribute SemQL expression to the query only if
				// it is valid for a Golden View.
				if (dataEditionConnection.getParser().parseSelectExpression(
						goldenView, formAttribute.getSemQLExpression()) != null) {
					qry.addSelectExpression(formAttribute.getSemQLExpression());
				}
			}
		}
		// add the PK attribute to the query (even if it is not in the form view).
		// This may be required to link the child records
		qry.addSelectExpression(entity.getPKAttribute().getName());

		

		// Running the query
		List<IEntityRow> rows = dataEditionConnection.getDataAccessService()
				.queryEntityRows(qry);

		// Dumping the resulting data in a text hierarchy.
		if (!rows.isEmpty()) {
			for (IEntityRow dataRow : rows) {
				indent(builder, level).append(boEntity.getName()).append(": ").append(dataRow.getValue(AttributeNames.goldenIdName(dataRow.getView()))).append("\n" );
				// Form Section
				boolean showSections = true;
				for (IFormSection formSection : formView.getFormSections()) {
					if (showSections) {
						indent(builder, level+1).append("section: ").append(formSection.getName()).append("\n");
					}
					for (IDisplayViewAttribute formAttribute : formSection
							.getFormAttributes()) {
						if (dataEditionConnection.getParser()
								.parseSelectExpression(goldenView,
										formAttribute.getSemQLExpression()) != null) {
							
							indent(builder, level+(showSections?2:1)).append(formAttribute.getName()).append(": ").append(dataRow.getValue(formAttribute.getSemQLExpression())).append("\n");
						}
					}
				}
				
				// Processing the child transitions
				for (IBOTransition boTransition : boEntity.getTransitions()) {
					indent(builder, level+1).append(boTransition.getName() + ":\n");
					// Building the reference filtering.													
					processBoEntity(dataEditionConnection,
							boTransition.getTargetBOEntity(),
							boTransition.getFilter(), boView, boTransition, dataRow, level +2, builder );					

				}

			}
		}
	}

	private static StringBuilder indent(StringBuilder builder, int pLevel) {
		for (int i= 0 ;i < pLevel; i++) {
			builder.append("  ");	
		}
		return builder;
	}
}
